﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Diagnostics;

namespace howto_count_inversions
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnGenerate_Click(object sender, EventArgs e)
        {
            Random rand = new Random();

            int length = int.Parse(txtLength.Text);
            string[] values = new string[length];
            for (int i = 0; i < length; i++)
                values[i] = rand.Next(10000).ToString();

            txtList.Text = string.Join(", ", values);
        }

        private void btnSolve_Click(object sender, EventArgs e)
        {
            txtN2Count.Clear();
            txtN2Time.Clear();
            txtNlogNCount.Clear();
            txtNlogNTime.Clear();
            Cursor = Cursors.WaitCursor;
            Refresh();

            Stopwatch watch = new Stopwatch();
            int[] values = LoadArray();
            watch.Start();
            txtN2Count.Text = N2Count(values).ToString();
            watch.Stop();
            txtN2Time.Text = watch.Elapsed.TotalSeconds.ToString("0.0000");

            values = LoadArray();
            watch.Reset();
            watch.Start();
            txtNlogNCount.Text = NlogNCount(values).ToString();
            watch.Stop();
            txtNlogNTime.Text = watch.Elapsed.TotalSeconds.ToString("0.0000");

            Cursor = Cursors.Default;
        }

        // Load the array of values.
        private int[] LoadArray()
        {
            return Array.ConvertAll<string, int>
                (txtList.Text.Split(','), int.Parse);
        }

        // Count inversions the simple O(N^2) way.
        private int N2Count(int[] values)
        {
            int count = 0;
            for (int i = 0; i < values.Length - 1; i++)
            {
                for (int j = i + 1; j < values.Length; j++)
                {
                    if (values[i] > values[j]) count++;
                }
            }
            return count;
        }

        private int NlogNCount(int[] values)
        {
            return MergesortAndCount(values, 0, values.Length - 1);
        }

        // Merge the pieces and return the number of inversions.
        private int MergeAndCount(int[] arr, int l, int m, int r)
        {
            int left_length = m - l + 1;
            int[] left = new int[left_length];
            Array.Copy(arr, l, left, 0, left_length);

            int right_length = r - (m + 1) + 1;
            int[] right = new int[right_length];
            Array.Copy(arr, m + 1, right, 0, right_length);

            int i = 0;
            int j = 0;
            int k = l;
            int count = 0;

            while (i < left_length && j < right_length)
            {
                if (left[i] <= right[j])
                {
                    arr[k++] = left[i++];
                }
                else
                {
                    arr[k++] = right[j++];
                    count += left_length - i;
                }
            }

            if (i < left_length)
            {
                Array.Copy(left, i, arr, k, left_length - i);
            }
            if (j < right_length)
            {
                Array.Copy(right, j, arr, k, right_length - j);
            }
    
            return count;
        }

        private int MergesortAndCount(int[] arr, int l, int r)
        {
            int count = 0;
            if (l < r)
            {
                int m = (l + r) / 2;
                count += MergesortAndCount(arr, l, m);
                count += MergesortAndCount(arr, m + 1, r);
                count += MergeAndCount(arr, l, m, r);
            }
            return count;
        }
    }
}
